/* $Id: planetdlg.c,v 1.1 2000/01/29 19:12:09 rsmith Exp $
 * This file is part of xplanets
 * Copyright (C) 2000  R.F. Smith
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * You can contact the author at the following addresses:
 *     e-mail: rsmith@xs4all.nl
 * snail-mail: R.F. Smith
 *             Dr. Hermansweg 36
 *             5624 HR Eindhoven
 *             The Netherlands
 *
 * $Log: planetdlg.c,v $
 * Revision 1.1  2000/01/29 19:12:09  rsmith
 * Initial revision
 *
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <gtk/gtk.h>
#include "planets.h"

#define LABELSIZE 64
#define YEAR_LEN 365.26

/*************** EXPORTS **************/ 
void show_planet_dialog (int i);

/************ STATIC DATA *************/
static GtkWidget *planet_dialog = NULL;
/*  static GtkWidget *data_frame = NULL; */
static GtkWidget *data_vbox = NULL;
/*  static GtkWidget * = NULL; */
static GtkWidget *mass_label = NULL;
static GtkWidget *radius_label = NULL;
static GtkWidget *orbit_label = NULL;
static GtkWidget *angle_label = NULL;
static GtkWidget *name_label = NULL;
static GtkWidget *period_label = NULL;

/********** STATIC FUNCTIONS **********/
static void create_planet_dialog (int i);
static void label_printf (GtkLabel *label, const char *fmt, ...);
static void delete_planet_dialog (GtkWidget *widget, gpointer *data);
static void update_planet_dialog (int i);

/*********** IMPLEMENTATION ***********/
static void 
create_planet_dialog (int i)
{
  char *txt = "Planet data";
  GtkWidget *data_frame = NULL;
  GtkWidget *close_btn = NULL;

  planet_dialog = gtk_window_new (GTK_WINDOW_DIALOG); 
  gtk_window_set_title(GTK_WINDOW (planet_dialog), txt);
  gtk_window_set_wmclass (GTK_WINDOW (planet_dialog),
                          txt, PROGRAM);
  gtk_window_set_position(GTK_WINDOW (planet_dialog), 
                          GTK_WIN_POS_CENTER);

  data_frame = gtk_frame_new (txt);
  gtk_container_border_width(GTK_CONTAINER (data_frame), 2);
  gtk_container_add (GTK_CONTAINER(planet_dialog), data_frame);

  data_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_border_width(GTK_CONTAINER (data_vbox), 2);
  gtk_container_add (GTK_CONTAINER (data_frame), data_vbox);
  
  update_planet_dialog (i);

  close_btn = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(data_vbox), close_btn, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(close_btn), "clicked", 
                      GTK_SIGNAL_FUNC(delete_planet_dialog), NULL);

  gtk_widget_show_all (planet_dialog);
}

static void
label_printf (GtkLabel *label, const char *fmt, ...)
{
  char buf[LABELSIZE];

  if (label != NULL && fmt != NULL) {
    va_list ap;
    va_start (ap, fmt);
    vsnprintf (buf, LABELSIZE, fmt, ap);
    gtk_label_set_text (label, buf);
    va_end (ap);
  }
}

static void 
delete_planet_dialog (GtkWidget *widget, gpointer *data)
{
  gtk_widget_destroy(planet_dialog);
  planet_dialog = NULL; 
  data_vbox = NULL;
  mass_label = NULL;
  radius_label = NULL;
  orbit_label = NULL;
  angle_label = NULL;
  name_label = NULL;
  period_label = NULL;
}

static void 
update_planet_dialog (int i)
{
  double period = 2*M_PI/(planet[i].w*3600*24); /* days */
  char *empty = "empty";
  char *unit[2] = {"days", "years"};

  if (name_label == NULL) {
    name_label = gtk_label_new (empty);
    gtk_box_pack_start(GTK_BOX(data_vbox), name_label, FALSE, FALSE, 0);
  }
  label_printf (GTK_LABEL (name_label), "%s", planet[i].name);

  if (mass_label == NULL) {
    mass_label = gtk_label_new (empty);
    gtk_box_pack_start(GTK_BOX(data_vbox), mass_label, FALSE, FALSE, 0);
  }
  label_printf (GTK_LABEL (mass_label), "planetary mass: %g kg", 
                planet[i].mass);

  if (radius_label == NULL) {
    radius_label = gtk_label_new (empty);
    gtk_box_pack_start(GTK_BOX(data_vbox), radius_label, FALSE, FALSE, 0);
  }
  label_printf (GTK_LABEL (radius_label), "planetary radius: %g km", 
                planet[i].radius/1000.0);

  if (orbit_label == NULL) { 
    orbit_label = gtk_label_new (empty);
    gtk_box_pack_start(GTK_BOX(data_vbox), orbit_label, FALSE, FALSE, 0);
  }
  label_printf (GTK_LABEL (orbit_label), "average orbital radius: %g km", 
                planet[i].orbit/1000.0);  

  if (period_label == NULL) { 
    period_label = gtk_label_new (empty);
    gtk_box_pack_start(GTK_BOX(data_vbox), period_label, FALSE, FALSE, 0);
  }
  if (period <= YEAR_LEN) {    
    label_printf (GTK_LABEL (period_label), "orbital period: %g %s", 
                  period, unit[0]);
  } else {
    label_printf (GTK_LABEL (period_label), "orbital period: %g %s", 
                  period/YEAR_LEN, unit[1]);
  }
}

void 
show_planet_dialog (int i)
{
  if (planet_dialog == NULL) {
    create_planet_dialog (i);
  } else {
    update_planet_dialog (i);
  }
}


